<?php

namespace Hnb\Wallet\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hnb\Wallet\Http\Requests\TransactionRequest;
use Hnb\Wallet\Interfaces\TransactionRepositoryInterface;
use Hnb\Wallet\Models\Transaction;

/**
 * Resource controller class for transaction.
 */
class TransactionResourceController extends BaseController
{

    /**
     * Initialize transaction resource controller.
     *
     * @param type TransactionRepositoryInterface $transaction
     *
     * @return null
     */
    public function __construct(TransactionRepositoryInterface $transaction)
    {
        parent::__construct();
        $this->repository = $transaction;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hnb\Wallet\Repositories\Criteria\TransactionResourceCriteria::class);
    }

    /**
     * Display a list of transaction.
     *
     * @return Response
     */
    public function index(TransactionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hnb\Wallet\Repositories\Presenter\TransactionPresenter::class)
                ->$function();
        }

        $transactions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('wallet::transaction.names'))
            ->view('wallet::transaction.index', true)
            ->data(compact('transactions', 'view'))
            ->output();
    }

    /**
     * Display transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function show(TransactionRequest $request, Transaction $transaction)
    {

        if ($transaction->exists) {
            $view = 'wallet::transaction.show';
        } else {
            $view = 'wallet::transaction.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('wallet::transaction.name'))
            ->data(compact('transaction'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TransactionRequest $request)
    {

        $transaction = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('wallet::transaction.name')) 
            ->view('wallet::transaction.create', true) 
            ->data(compact('transaction'))
            ->output();
    }

    /**
     * Create new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TransactionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $transaction                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('wallet::transaction.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('wallet/transaction/' . $transaction->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/wallet/transaction'))
                ->redirect();
        }

    }

    /**
     * Show transaction for editing.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function edit(TransactionRequest $request, Transaction $transaction)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('wallet::transaction.name'))
            ->view('wallet::transaction.edit', true)
            ->data(compact('transaction'))
            ->output();
    }

    /**
     * Update the transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function update(TransactionRequest $request, Transaction $transaction)
    {
        try {
            $attributes = $request->all();

            $transaction->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('wallet::transaction.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('wallet/transaction/' . $transaction->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('wallet/transaction/' . $transaction->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the transaction.
     *
     * @param Model   $transaction
     *
     * @return Response
     */
    public function destroy(TransactionRequest $request, Transaction $transaction)
    {
        try {

            $transaction->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('wallet::transaction.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('wallet/transaction/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('wallet/transaction/' . $transaction->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple transaction.
     *
     * @param Model   $transaction
     *
     * @return Response
     */
    public function delete(TransactionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('wallet::transaction.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('wallet/transaction'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/wallet/transaction'))
                ->redirect();
        }

    }

    /**
     * Restore deleted transactions.
     *
     * @param Model   $transaction
     *
     * @return Response
     */
    public function restore(TransactionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('wallet::transaction.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/wallet/transaction'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/wallet/transaction/'))
                ->redirect();
        }

    }

}
