<?php

namespace Hnb\Wallet\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hnb\Wallet\Http\Requests\WalletRequest;
use Hnb\Wallet\Interfaces\WalletRepositoryInterface;
use Hnb\Wallet\Models\Wallet;

/**
 * Resource controller class for wallet.
 */
class WalletResourceController extends BaseController
{

    /**
     * Initialize wallet resource controller.
     *
     * @param type WalletRepositoryInterface $wallet
     *
     * @return null
     */
    public function __construct(WalletRepositoryInterface $wallet)
    {
        parent::__construct();
        $this->repository = $wallet;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hnb\Wallet\Repositories\Criteria\WalletResourceCriteria::class);
    }

    /**
     * Display a list of wallet.
     *
     * @return Response
     */
    public function index(WalletRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hnb\Wallet\Repositories\Presenter\WalletPresenter::class)
                ->$function();
        }

        $wallets = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('wallet::wallet.names'))
            ->view('wallet::wallet.index', true)
            ->data(compact('wallets', 'view'))
            ->output();
    }

    /**
     * Display wallet.
     *
     * @param Request $request
     * @param Model   $wallet
     *
     * @return Response
     */
    public function show(WalletRequest $request, Wallet $wallet)
    {

        if ($wallet->exists) {
            $view = 'wallet::wallet.show';
        } else {
            $view = 'wallet::wallet.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('wallet::wallet.name'))
            ->data(compact('wallet'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new wallet.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(WalletRequest $request)
    {

        $wallet = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('wallet::wallet.name')) 
            ->view('wallet::wallet.create', true) 
            ->data(compact('wallet'))
            ->output();
    }

    /**
     * Create new wallet.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(WalletRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $wallet                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('wallet::wallet.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('wallet/wallet/' . $wallet->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/wallet/wallet'))
                ->redirect();
        }

    }

    /**
     * Show wallet for editing.
     *
     * @param Request $request
     * @param Model   $wallet
     *
     * @return Response
     */
    public function edit(WalletRequest $request, Wallet $wallet)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('wallet::wallet.name'))
            ->view('wallet::wallet.edit', true)
            ->data(compact('wallet'))
            ->output();
    }

    /**
     * Update the wallet.
     *
     * @param Request $request
     * @param Model   $wallet
     *
     * @return Response
     */
    public function update(WalletRequest $request, Wallet $wallet)
    {
        try {
            $attributes = $request->all();

            $wallet->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('wallet::wallet.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('wallet/wallet/' . $wallet->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('wallet/wallet/' . $wallet->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the wallet.
     *
     * @param Model   $wallet
     *
     * @return Response
     */
    public function destroy(WalletRequest $request, Wallet $wallet)
    {
        try {

            $wallet->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('wallet::wallet.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('wallet/wallet/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('wallet/wallet/' . $wallet->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple wallet.
     *
     * @param Model   $wallet
     *
     * @return Response
     */
    public function delete(WalletRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('wallet::wallet.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('wallet/wallet'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/wallet/wallet'))
                ->redirect();
        }

    }

    /**
     * Restore deleted wallets.
     *
     * @param Model   $wallet
     *
     * @return Response
     */
    public function restore(WalletRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('wallet::wallet.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/wallet/wallet'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/wallet/wallet/'))
                ->redirect();
        }

    }

}
